/*
 * Create an entire Customer/Address/Account/Service/Payment hierarchy with
 * one instance of each with made-up values. Do all this inside a transaction
 * to show that it can be done.
 */
package TelcoClient;

import javax.ejb.*;
import javax.naming.*;
import javax.persistence.*;
import javax.transaction.UserTransaction;
import java.util.Date;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;

import TelcoEntity.*;
import TelcoSession.*;

public class NewService
{
  // The various facades used to invoke the bean methods.
  private static CustomerFacadeRemote customerFacade = null;
  private static AddressFacadeRemote  addressFacade  = null;
  private static AccountFacadeRemote  accountFacade  = null;
  private static ServiceFacadeRemote  serviceFacade  = null;
  private static PaymentFacadeRemote  paymentFacade  = null;

  // The user transaction in which everything is wrapped.
  //ivate static EntityTransaction transaction = null;
  private static UserTransaction transaction = null;

  // The various created entities used for foreign key assignments.
  private static Customer customer = null;
  private static Address  address  = null;
  private static Account  account  = null;
  private static Service  service  = null;
  private static Payment  payment  = null;

  public static void main(String[] args) throws Exception
  {
    Context ic = new InitialContext();

    createFacades(ic);
    createUserTransaction(ic, "java:comp/UserTransaction"); // glassfish

    transaction.begin();
    createCustomer();
    createAddress();
    createAccount();
    createService();
    createPayment();

    System.out.println("Transaction status is: " + transaction.getStatus());

    if (args.length == 0)
    {
      transaction.commit();
    }
    else
    {
      transaction.rollback();
    }

    System.out.println("Transaction status is: " + transaction.getStatus());
  }

  private static void createFacades(Context ic)
  {
    Object obj;

    try
    {
      obj = ic.lookup(CustomerFacadeRemote.class.getName());
      customerFacade = (CustomerFacadeRemote) obj;

      obj = ic.lookup(AddressFacadeRemote.class.getName());
      addressFacade = (AddressFacadeRemote) obj;

      obj = ic.lookup(AccountFacadeRemote.class.getName());
      accountFacade = (AccountFacadeRemote) obj;

      obj = ic.lookup(ServiceFacadeRemote.class.getName());
      serviceFacade = (ServiceFacadeRemote) obj;

      obj = ic.lookup(PaymentFacadeRemote.class.getName());
      paymentFacade = (PaymentFacadeRemote) obj;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.err.println("ERROR in lookup of facades: " + e.getMessage());
      System.exit(1);
    }
  }

  private static void createUserTransaction(Context ic, String jndiName)
  {
    try
    {
      //EntityManagerFactory entityManagerFactory =
      //  Persistence.createEntityManagerFactory("ExampleEJB3xModulePU");
      //EntityManager em = entityManagerFactory.createEntityManager();
      //transaction = em.getTransaction();

      transaction = (UserTransaction) ic.lookup(jndiName);

      if (transaction == null)
      {
        System.err.println("transaction is NULL!");
        System.exit(1);
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.err.println("ERROR in lookup of UserTransaction by JNDI name '"+
                         jndiName + "':");
      System.err.println(e.getMessage());
      System.exit(1);
    }
  }

  private static void createCustomer()
  {
    customer = new Customer();
    customer.setName("Glassfish");

    System.out.println("Create Customer:");
    System.out.println("    name = " + customer.getName());

    customerFacade.create(customer);
  }

  private static void createAddress()
  {
    AddressPK addressPK = new AddressPK();
    addressPK.setCname(customer.getName());
    addressPK.setLine1("1 Elm St");

    address = new Address();
    address.setAddressPK(addressPK);
    address.setCity("New York");
    address.setState("NY");
    address.setSuburbCode("12345");
    address.setCountry("United States");
    address.setTelephone("+1-555-555-5555");
    address.setEmail("info@example.com");

    System.out.println("Create Address:");
    System.out.println("    customer = " + address.getAddressPK().getCname());
    System.out.println("    line1 = " + address.getAddressPK().getLine1());
    System.out.println("    city = " + address.getCity());
    System.out.println("    state = " + address.getState());
    System.out.println("    suburbCode = " + address.getSuburbCode());
    System.out.println("    country = " + address.getCountry());
    System.out.println("    telephone = " + address.getTelephone());
    System.out.println("    email = " + address.getEmail());

    addressFacade.create(address);
  }

  private static void createAccount()
  {
    account = new Account();
    account.setAccnum(1000000 + (int) (java.lang.Math.random() * 1000));
    account.setCname(customer);
    account.setAddress(address);
    account.setDiscount(Short.valueOf((short) 10));
    account.setRating('H');
    account.setBalance(BigDecimal.valueOf(100.0));

    System.out.println("Create Account:");
    System.out.println("    accnum = " + account.getAccnum());
    System.out.println("    customer = " + account.getCname());
    System.out.println("    address = " + account.getAddress()
                                                 .getAddressPK()
                                                 .getLine1());
    System.out.println("    discount = " + account.getDiscount());
    System.out.println("    rating = " + account.getRating());
    System.out.println("    balance = " + account.getBalance());

    accountFacade.create(account);
  }

  private static void createService()
  {
    service = new Service();
    service.setSvcno("+1-555-555-5555");
    service.setAccnum(account);
    service.setRate(Short.valueOf((short) 5));
    service.setStype('F');

    System.out.println("Create Service:");
    System.out.println("    svcno = " + service.getSvcno());
    System.out.println("    accnum = " + service.getAccnum());
    System.out.println("    rate = " + service.getRate());
    System.out.println("    type = " + service.getStype());

    serviceFacade.create(service);
  }

  private static void createPayment()
  {
    payment = new Payment();
    payment.setAccnum(account);
    payment.setAmount(BigDecimal.valueOf(100.0));
    payment.setPaidOn(parseDate("2008-01-01"));

    System.out.println("Create Payment:");
    System.out.println("    uid = " + payment.getUid());
    System.out.println("    accnum = " + payment.getAccnum());
    System.out.println("    amount = " + payment.getAmount());
    System.out.println("    date = " + payment.getPaidOn());

    paymentFacade.create(payment);
  }

  private static void setRollbackOnly()
  {
    try
    {
      transaction.setRollbackOnly();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.err.println("ERROR setting transaction to rollback only: " +
                         e.getMessage());
      System.exit(1);
    }
  }

  private static Date parseDate(String sDate)
  {
    Date d = null;
    try
    {
      d = new SimpleDateFormat("yyyy-MM-dd").parse(sDate);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.err.println("ERROR parsing date: " + e.getMessage());
      System.exit(1);
    }

    return d;
  }
}
